package feynmanplate;

import org.opensourcephysics.numerics.*;
import org.opensourcephysics.displayejs.*;
import java.awt.Color;
import java.awt.Font;

/**
 * <p>Title: Plate</p>
 * <p>Description: Subclass of RigidBody. It can draw itself and compute its motion.</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * @author Alexander Tomori, Boris Gazovic, Slavomir Tuleja
 * @version 0.01
 */
public class Plate extends RigidBody {
  int nRadialPl=16;//for plate
  int nAzimuthalPl=36;
  int nRadialMed=2;//for medallion
  int nAzimuthalMed=10;
  double thickness=0.05;
  double radius=0.75;

  double[][][] dataPl=new double[nRadialPl][nAzimuthalPl][3];//original plate
  double[][][] dataPlTransformed=new double[nRadialPl][nAzimuthalPl][3];//transformed plate
  double[][][] dataMed=new double[nRadialMed][nAzimuthalMed][3];//original medallion
  double[][][] dataMedTransformed=new double[nRadialMed][nAzimuthalMed][3];//trnasformed medallion

  InteractiveSurface surfacePl, surfaceMed;//surfaces of plate and medallion

  /**
   * Constructor of the plate.
   */
  public Plate(){
    super(0.03242,0.06172);

    //computeX3CoordinateOfCenterOfMass();
    //computeI3();
    //computeI();

    //Create medallion InteractiveSurface
    surfaceMed=new InteractiveSurface();
    surfaceMed.setColorOriginAndDirection(0.0, 0.0, 0.0, 0.0, 0.0, 1.0);
    surfaceMed.setColorRegions(new double[]{}, new Color[]{Color.red});
    surfaceMed.getStyle().setEdgeColor(Color.RED);
    surfaceMed.setDisplacementFactor(0.9);

    //Create medallion data
    for(int i=0; i<nRadialMed; i++){
      for(int j=0; j<nAzimuthalMed; j++){
        double r=0.06 * i /(nRadialMed-1);
        double angle=2*Math.PI*j/(nAzimuthalMed-1);

        double x=r*Math.cos(angle)+0.9*radius;
        double y=r*Math.sin(angle);
        dataMed[i][j][0]=x;
        dataMed[i][j][1]=y;
        dataMed[i][j][2]=generatingFunctionValue(Math.sqrt(x*x+y*y))+thickness/2-0.072;
      }
    }

    surfaceMed.setData(dataMed);

    //Create plate InteractiveSurface
    surfacePl=new InteractiveSurface();
    surfacePl.getStyle().setEdgeColor(new Color(53, 97, 255));
    surfacePl.setColorOriginAndDirection(0.0, 0.0, 0.0, 0.0, 0.0, 5.0);
    surfacePl.setColorRegions(new double[]{}, new Color[]{new Color(198, 222, 255)});

    //Create plate data
    for(int i=0; i<nRadialPl; i++){
      for(int j=0; j<nAzimuthalPl; j++){
        double r, angle;
        if(i<nRadialPl/2){
          r = radius * i / (nRadialPl/2 - 1);
          angle = 2 * Math.PI * j / (nAzimuthalPl - 1);
          dataPl[i][j][0] = r * Math.cos(angle);
          dataPl[i][j][1] = r * Math.sin(angle);
          dataPl[i][j][2] = generatingFunctionValue(r)-thickness/2-0.072;
        }
        else {
          r = -radius * i / (nRadialPl/2 - 1) + radius*(nRadialPl-1)/(nRadialPl/2-1);
          angle = 2 * Math.PI * j / (nAzimuthalPl - 1);
          dataPl[i][j][0] = r * Math.cos(angle);
          dataPl[i][j][1] = r * Math.sin(angle);
          dataPl[i][j][2] = generatingFunctionValue(r)+thickness/2-0.072;
        }
      }
    }

    surfacePl.setData(dataPl);

    add(surfacePl);
    add(surfaceMed);

    setVisibility();
  }

  /**
   * Computes the x3-coordinate of Center of Mass.
   * @return
   */
  private double computeX3CoordinateOfCenterOfMass(){
    double x3CM;
    double r, angle, x, y, z, m, sumNom=0, sumDenom=0;
    int nRadialPl=500;
    int nAzimuthalPl=360;


    for (int i = 0; i < nRadialPl; i++) {
      for (int j = 0; j < nAzimuthalPl; j++) {
        r = radius * (i+0.5) / (nRadialPl - 1);
        angle = 2 * Math.PI * (j+0.5) / (nAzimuthalPl - 1);
        x = r * Math.cos(angle);
        y = r * Math.sin(angle);
        z = generatingFunctionValue(r);
        m=radius/(nRadialPl-1) * r * 2*Math.PI/(nAzimuthalPl - 1) * Math.sqrt(1+derivative(r)*derivative(r));
        sumNom+=m*z;
        sumDenom+=m;
      }
    }
    x3CM=sumNom/sumDenom;
    System.out.println("CM "+x3CM);
    return x3CM;
  }

  /**
   * Computes the I3
   * @return
   */
  private double computeI3(){
    double I3;
    double r, angle, x, y, z, m, sum=0;
    int nRadialPl=500;
    int nAzimuthalPl=360;


    for (int i = 0; i < nRadialPl; i++) {
      for (int j = 0; j < nAzimuthalPl; j++) {
        r = radius * (i+0.5) / (nRadialPl - 1);
        angle = 2 * Math.PI * (j+0.5) / (nAzimuthalPl - 1);
        x = r * Math.cos(angle);
        y = r * Math.sin(angle);
        z = generatingFunctionValue(r);
        m=0.12*radius/(nRadialPl-1) * r * 2*Math.PI/(nAzimuthalPl - 1) * Math.sqrt(1+derivative(r)*derivative(r));
        sum+=m*(x*x+y*y);
      }
    }
    I3=sum;
    System.out.println("I3 "+I3);
    return I3;
  }

  /**
     * Computes the I.
     * @return
     */
    private double computeI(){
      double I;
      double r, angle, x, y, z, m, sum=0;
      int nRadialPl=500;
      int nAzimuthalPl=360;

      for (int i = 0; i < nRadialPl; i++) {
        for (int j = 0; j < nAzimuthalPl; j++) {
          r = radius * (i+0.5) / (nRadialPl - 1);
          angle = 2 * Math.PI * (j+0.5) / (nAzimuthalPl - 1);
          x = r * Math.cos(angle);
          y = r * Math.sin(angle);
          z = generatingFunctionValue(r);
          m=0.12*radius/(nRadialPl-1) * r * 2*Math.PI/(nAzimuthalPl - 1) * Math.sqrt(1+derivative(r)*derivative(r));
          sum+=m*(x*x+z*z);
        }
      }
      I=sum;
      System.out.println("I "+I);
      return I;
    }



  public void setVisibility(){
    super.setVisibilityExceptRigidBody();
    surfacePl.setVisible(showRigidBody);
    surfaceMed.setVisible(showRigidBody);
  }

  public void transformBodyOnly(double[][] rm){
    for (int i = 0; i < nRadialPl; i++) {
     for (int j = 0; j < nAzimuthalPl; j++) {
       double x = dataPl[i][j][0];
       double y = dataPl[i][j][1];
       double z = dataPl[i][j][2];

       double xn = rm[0][0] * x + rm[0][1] * y + rm[0][2] * z;
       double yn = rm[1][0] * x + rm[1][1] * y + rm[1][2] * z;
       double zn = rm[2][0] * x + rm[2][1] * y + rm[2][2] * z;

       dataPlTransformed[i][j][0] = xn;
       dataPlTransformed[i][j][1] = yn;
       dataPlTransformed[i][j][2] = zn;
     }
   }
   surfacePl.setData(dataPlTransformed);

   //transforms medallion
   for (int i = 0; i < nRadialMed; i++) {
     for (int j = 0; j < nAzimuthalMed; j++) {
       double x = dataMed[i][j][0];
       double y = dataMed[i][j][1];
       double z = dataMed[i][j][2];

       double xn = rm[0][0] * x + rm[0][1] * y + rm[0][2] * z;
       double yn = rm[1][0] * x + rm[1][1] * y + rm[1][2] * z;
       double zn = rm[2][0] * x + rm[2][1] * y + rm[2][2] * z;

       dataMedTransformed[i][j][0] = xn;
       dataMedTransformed[i][j][1] = yn;
       dataMedTransformed[i][j][2] = zn;
     }
   }
   surfaceMed.setData(dataMedTransformed);
  }

  /**
   * This function defines the shape of the plate.
   * @param r distnce to the center, shold be from 0.0 to radius of the plate
   * @return z z-coordinate of the given point
   */
  private double generatingFunctionValue(double r){
    double offset=radius*0.3;//the radius of the base of the plate
    double z=0.0;
    if(r>=offset) z=0.3*Math.log(r-offset+1.0);
    return z;
  }

  /**
   * Returns the derivative of generating function.
   * @param r radial coordinate
   * @return derivative of generating function at radial coordinate r
   */
  private double derivative(double r){
    double offset=radius*0.3;//the radius of the base of the plate
    double der=0.0;
    if(r>=offset) der=0.3/(r-offset+1.0);
    return der;
  }

}